library(effsize)

results <- read.csv("data.csv")

########################################################################################################################
#CORRECT vs. INCORRECT
correct <- results[which(results$correctness == 1.0),]
incorrect <- results[which(results$correctness != 1.0),]

pvalues <- NULL

t <- wilcox.test(correct$alerting, incorrect$alerting, paired = FALSE)
t
pvalues <- c(pvalues, t$p.value)
cliff.delta(correct$alerting, incorrect$alerting)

t <- wilcox.test(correct$orienting, incorrect$orienting, paired = FALSE)
t
pvalues <- c(pvalues, t$p.value)
cliff.delta(correct$orienting, incorrect$orienting)

t <- wilcox.test(correct$executive_control, incorrect$executive_control, paired = FALSE)
t
pvalues <- c(pvalues, t$p.value)
cliff.delta(correct$executive_control, incorrect$executive_control)

t <- wilcox.test(correct$immediate_recall, incorrect$immediate_recall, paired = FALSE)
t
pvalues <- c(pvalues, t$p.value)
cliff.delta(correct$immediate_recall, incorrect$immediate_recall)

t <- wilcox.test(correct$working_memory, incorrect$working_memory, paired = FALSE)
t
pvalues <- c(pvalues, t$p.value)
cliff.delta(correct$working_memory, incorrect$working_memory)

t <- wilcox.test(correct$programming_experience, incorrect$programming_experience, paired = FALSE)
t
pvalues <- c(pvalues, t$p.value)
cliff.delta(correct$programming_experience, incorrect$programming_experience)

number_correct_easy <- length(which(correct$task_difficulty == "easy"))
number_correct_difficult <- length(which(correct$task_difficulty == "hard"))

number_incorrect_easy <- length(which(incorrect$task_difficulty == "easy"))
number_incorrect_difficult <- length(which(incorrect$task_difficulty == "hard"))

correctness_task_difficulty_matrix <- matrix(c(number_correct_easy, number_incorrect_easy, number_correct_difficult, number_incorrect_difficult), nrow = 2, ncol = 2)
t <- fisher.test(correctness_task_difficulty_matrix)
t
pvalues <- c(pvalues, t$p.value)

number_correct_bug_fixing <- length(which(correct$task_type == "bug_fixing"))
number_correct_feature_implementation <- length(which(correct$task_type == "feature_implementation"))

number_incorrect_bug_fixing <- length(which(incorrect$task_type == "bug_fixing"))
number_incorrect_feature_implementation <- length(which(incorrect$task_type == "feature_implementation"))

correctness_task_type_matrix <- matrix(c(number_correct_bug_fixing, number_correct_feature_implementation, number_incorrect_bug_fixing, number_incorrect_feature_implementation), nrow = 2, ncol = 2)
t <- fisher.test(correctness_task_type_matrix)
t
pvalues <- c(pvalues, t$p.value)

p.adjust(pvalues, method = "BH")

########################################################################################################################
#TIME-UP vs. NON-TIME-UP
time_is_up <- results[which(results$time == 30),]
time_is_not_up <- results[which(results$time < 30),]

pvalues <- NULL

t <- wilcox.test(time_is_up$alerting, time_is_not_up$alerting, paired = FALSE)
t
pvalues <- c(pvalues, t$p.value)
cliff.delta(time_is_up$alerting, time_is_not_up$alerting)

t <- wilcox.test(time_is_up$orienting, time_is_not_up$orienting, paired = FALSE)
t
pvalues <- c(pvalues, t$p.value)
cliff.delta(time_is_up$orienting, time_is_not_up$orienting)

t <- wilcox.test(time_is_up$executive_control, time_is_not_up$executive_control, paired = FALSE)
t
pvalues <- c(pvalues, t$p.value)
cliff.delta(time_is_up$executive_control, time_is_not_up$executive_control)

t <- wilcox.test(time_is_up$immediate_recall, time_is_not_up$immediate_recall, paired = FALSE)
t
pvalues <- c(pvalues, t$p.value)
cliff.delta(time_is_up$immediate_recall, time_is_not_up$immediate_recall)

t <- wilcox.test(time_is_up$working_memory, time_is_not_up$working_memory, paired = FALSE)
t
pvalues <- c(pvalues, t$p.value)
cliff.delta(time_is_up$working_memory, time_is_not_up$working_memory)

t <- wilcox.test(time_is_up$programming_experience, time_is_not_up$programming_experience, paired = FALSE)
t
pvalues <- c(pvalues, t$p.value)
cliff.delta(time_is_up$programming_experience, time_is_not_up$programming_experience)

number_time_is_up_easy <- length(which(time_is_up$task_difficulty == "easy"))
number_time_is_up_difficult <- length(which(time_is_up$task_difficulty == "hard"))

number_time_is_not_up_easy <- length(which(time_is_not_up$task_difficulty == "easy"))
number_time_is_not_up_difficult <- length(which(time_is_not_up$task_difficulty == "hard"))

time_difficulty_matrix <- matrix(c(number_time_is_up_easy, number_time_is_not_up_easy, number_time_is_up_difficult, number_time_is_not_up_difficult), nrow = 2, ncol = 2)
t <- fisher.test(time_difficulty_matrix)
t
pvalues <- c(pvalues, t$p.value)

number_time_is_up_bug_fixing <- length(which(time_is_up$task_type == "bug_fixing"))
number_time_is_up_feature_implementation <- length(which(time_is_up$task_type == "feature_implementation"))

number_time_is_not_up_bug_fixing <- length(which(time_is_not_up$task_type == "bug_fixing"))
number_time_is_not_up_feature_implementation <- length(which(time_is_not_up$task_type == "feature_implementation"))

task_type_matrix <- matrix(c(number_time_is_up_bug_fixing, number_time_is_not_up_bug_fixing, number_time_is_up_feature_implementation, number_time_is_not_up_feature_implementation), nrow = 2, ncol = 2)
t <- fisher.test(task_type_matrix)
t
pvalues <- c(pvalues, t$p.value)

p.adjust(pvalues, method = "BH")


########################################################################################################################
#READABLE vs. UNREADABLE
readable <- feature_implementation[which(feature_implementation$readability >= 0.5),]
unreadable <- feature_implementation[which(feature_implementation$readability < 0.5),]

pvalues <- NULL

t <- wilcox.test(readable$alerting, unreadable$alerting, paired = FALSE)
t
pvalues <- c(pvalues, t$p.value)
cliff.delta(readable$alerting, unreadable$alerting)

t <- wilcox.test(readable$orienting, unreadable$orienting, paired = FALSE)
t
pvalues <- c(pvalues, t$p.value)
cliff.delta(readable$orienting, unreadable$orienting)

t <- wilcox.test(readable$executive_control, unreadable$executive_control, paired = FALSE)
t
pvalues <- c(pvalues, t$p.value)
cliff.delta(readable$executive_control, unreadable$executive_control)

t <- wilcox.test(readable$immediate_recall, unreadable$immediate_recall, paired = FALSE)
t
pvalues <- c(pvalues, t$p.value)
cliff.delta(readable$immediate_recall, unreadable$immediate_recall)

t <- wilcox.test(readable$working_memory, unreadable$working_memory, paired = FALSE)
t
pvalues <- c(pvalues, t$p.value)
cliff.delta(readable$working_memory, unreadable$working_memory)

t <- wilcox.test(readable$programming_experience, unreadable$programming_experience, paired = FALSE)
t
pvalues <- c(pvalues, t$p.value)
cliff.delta(readable$programming_experience, unreadable$programming_experience)

number_readable_easy <- length(which(readable$task_difficulty == "easy"))
number_readable_difficult <- length(which(readable$task_difficulty == "hard"))

number_unreadable_easy <- length(which(unreadable$task_difficulty == "easy"))
number_unreadable_difficult <- length(which(unreadable$task_difficulty == "hard"))

time_difficulty_matrix <- matrix(c(number_readable_easy, number_unreadable_easy, number_readable_difficult, number_unreadable_difficult), nrow = 2, ncol = 2)
t <- fisher.test(time_difficulty_matrix)
t
pvalues <- c(pvalues, t$p.value)

p.adjust(pvalues, method = "BH")

temp_feature_implementation <- feature_implementation
temp_feature_implementation$task_difficulty_int <- 0
for(i in 1:nrow(temp_feature_implementation)){
  if(temp_feature_implementation$task_difficulty[i] == "easy"){
    temp_feature_implementation$task_difficulty_int[i] <- 0
  } else {
    temp_feature_implementation$task_difficulty_int[i] <- 1
  }
}

temp_readable <- temp_feature_implementation[which(temp_feature_implementation$readability >= 0.5),]
temp_unreadable <- temp_feature_implementation[which(temp_feature_implementation$readability < 0.5),]
cliff.delta(temp_readable$task_difficulty_int, temp_unreadable$task_difficulty_int)


